"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _chalk = _interopRequireDefault(require("chalk"));

var fs = _interopRequireWildcard(require("fs"));

var _globalModules = _interopRequireDefault(require("global-modules"));

var logger = _interopRequireWildcard(require("../utils/logger"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function printHostingInstructions(appPackage, publicUrl, publicPath, buildFolder) {
  if (publicUrl && publicUrl.includes('.github.io/')) {
    // "homepage": "http://user.github.io/project"
    const publicPathname = new URL(publicPath).pathname;
    const hasDeployScript = typeof appPackage.scripts !== 'undefined' && typeof appPackage.scripts.deploy !== 'undefined';
    printBaseMessage(buildFolder, publicPathname);
    printDeployInstructions(publicUrl, hasDeployScript);
  } else if (publicPath !== '/') {
    // "homepage": "http://mywebsite.com/project"
    printBaseMessage(buildFolder, publicPath);
  } else {
    // "homepage": "http://mywebsite.com"
    //   or no homepage
    printBaseMessage(buildFolder, publicUrl);
    printStaticServerInstructions(buildFolder);
  }

  logger.log();
  logger.log('Find out more about deployment here:');
  logger.log();
  logger.log(`  ${_chalk.default.yellow('https://cra.link/deployment')}`);
  logger.log();
}

function printBaseMessage(buildFolder, hostingLocation) {
  logger.log(`The project was built assuming it is hosted at ${_chalk.default.green(hostingLocation || 'the server root')}.`);
  logger.log(`You can control this with the ${_chalk.default.green('homepage')} field in your ${_chalk.default.cyan('package.json')}.`);

  if (!hostingLocation) {
    logger.log('For example, add this to build it for GitHub Pages:');
    logger.log();
    logger.log(`  ${_chalk.default.green('"homepage"')} ${_chalk.default.cyan(':')} ${_chalk.default.green('"http://myname.github.io/myapp"')}${_chalk.default.cyan(',')}`);
  }

  logger.log();
  logger.log(`The ${_chalk.default.cyan(buildFolder)} folder is ready to be deployed.`);
}

function printDeployInstructions(publicUrl, hasDeployScript) {
  logger.log(`To publish it at ${_chalk.default.green(publicUrl)} , run:`);
  logger.log(); // If script deploy has been added to package.json, skip the instructions

  if (!hasDeployScript) {
    logger.log(`  ${_chalk.default.cyan('yarn')} add --dev gh-pages`);
    logger.log();
    logger.log(`Add the following script in your ${_chalk.default.cyan('package.json')}.`);
    logger.log();
    logger.log(`    ${_chalk.default.dim('// ...')}`);
    logger.log(`    ${_chalk.default.yellow('"scripts"')}: {`);
    logger.log(`      ${_chalk.default.dim('// ...')}`);
    logger.log(`      ${_chalk.default.yellow('"predeploy"')}: ${_chalk.default.yellow(`yarn build",`)}`);
    logger.log(`      ${_chalk.default.yellow('"deploy"')}: ${_chalk.default.yellow('"gh-pages -d build"')}`);
    logger.log('    }');
    logger.log();
    logger.log('Then run:');
    logger.log();
  }

  logger.log(`  ${_chalk.default.cyan('yarn')} run deploy`);
}

function printStaticServerInstructions(buildFolder) {
  logger.log('You may serve it with a static server:');
  logger.log();

  if (!fs.existsSync(`${_globalModules.default}/serve`)) {
    logger.log(`  ${_chalk.default.cyan('yarn')} global add serve`);
  }

  logger.log(`  ${_chalk.default.cyan('serve')} -s ${buildFolder}`);
}

var _default = printHostingInstructions;
exports.default = _default;
//# sourceMappingURL=printHostingInstructions.js.map