"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.check = check;

var _chalk = _interopRequireDefault(require("chalk"));

var _dedent = _interopRequireDefault(require("dedent"));

var fs = _interopRequireWildcard(require("fs-extra"));

var _semver = _interopRequireDefault(require("semver"));

var path = _interopRequireWildcard(require("path"));

var _semverRegex = _interopRequireDefault(require("semver-regex"));

var logger = _interopRequireWildcard(require("../utils/logger"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Based on https://github.com/facebook/create-react-app/blob/master/packages/react-scripts/scripts/utils/verifyPackageTree.js
 * Customized for modular
 */
const DEPS_TO_CHECK = [// These are packages most likely to break in practice.
'esbuild', 'eslint', 'webpack', 'webpack-dev-server']; // We assume that having wrong versions of these
// in the tree will likely break your setup.
// This is a relatively low-effort way to find common issues.

async function check() {
  // eslint-disable-next-line @typescript-eslint/no-var-requires
  const ownPackageJson = require('modular-scripts/package.json');

  const dependencies = ownPackageJson.dependencies || {};
  const expectedVersionsByDep = {}; // Gather wanted deps

  DEPS_TO_CHECK.forEach(dep => {
    const expectedVersion = dependencies[dep];

    if (!expectedVersion) {
      throw new Error('This dependency list is outdated, fix it.');
    }

    if (!(0, _semverRegex.default)().test(expectedVersion)) {
      throw new Error(`The ${dep} package should be pinned, instead got version ${expectedVersion}.`);
    }

    expectedVersionsByDep[dep] = expectedVersion;
  }); // Verify we don't have other versions up the tree

  const startDirectory = path.dirname(require.resolve('modular-scripts/package.json'));

  for (const dep of DEPS_TO_CHECK) {
    let currentDir = startDirectory; // We've reached the root of the modular repo
    // in which case we want to bail out

    while (currentDir !== path.resolve('/')) {
      currentDir = path.resolve(currentDir, '..');
      const maybeNodeModules = path.resolve(currentDir, 'node_modules');

      if (!fs.existsSync(maybeNodeModules)) {
        continue;
      }

      const maybeDep = path.resolve(maybeNodeModules, dep);

      if (!fs.existsSync(maybeDep)) {
        logger.debug(`${dep} is not installed in ${maybeNodeModules}.`);
        continue;
      }

      const maybeDepPackageJson = path.resolve(maybeDep, 'package.json');

      if (!fs.existsSync(maybeDepPackageJson)) {
        logger.debug(`${dep} does not have package.json in ${maybeDep}`);
        continue;
      }

      const packageJsonString = await fs.readFile(maybeDepPackageJson, 'utf8');
      const depPackageJson = JSON.parse(packageJsonString);
      const expectedVersion = expectedVersionsByDep[dep];

      if (_semver.default.satisfies(depPackageJson.version, expectedVersion)) {
        logger.debug(`${dep} is valid ${currentDir}@${depPackageJson.version}`);
        continue;
      } else {
        logger.debug(`${dep} is invalid ${currentDir}@${depPackageJson.version}`);
        logger.log((0, _dedent.default)(`\nThere might be a problem with the project dependency tree.\n` + `It is likely ${_chalk.default.bold('not')} a bug in modular-scripts, but something you need to fix locally.\n\n`) + `The ${_chalk.default.bold(ownPackageJson.name)} package provided by modular-scripts requires a dependency:\n\n` + _chalk.default.green(`  "${_chalk.default.bold(dep)}": "${_chalk.default.bold(expectedVersion)}"\n\n`) + `Don't try to install it manually: your package manager does it automatically.\n` + `However, a different version of ${_chalk.default.bold(dep)} was detected higher up in the tree:\n\n` + `  ${_chalk.default.bold(_chalk.default.red(maybeDep))} (version: ${_chalk.default.bold(_chalk.default.red(depPackageJson.version))}) \n\n` + `Manually installing incompatible versions is known to cause hard-to-debug issues.\n\n` + _chalk.default.red(`If you would prefer to ignore this check, add ${_chalk.default.bold('SKIP_PREFLIGHT_CHECK=true')} to an ${_chalk.default.bold('.env')} file in your project.\n` + `That will permanently disable this message but you might encounter other issues.\n\n`) + `To ${_chalk.default.green('fix')} the dependency tree, try following the steps below in the exact order:\n\n` + `  ${_chalk.default.cyan('1.')} Delete ${_chalk.default.bold('yarn.lock')} in your project folder.\n` + `  ${_chalk.default.cyan('2.')} Delete ${_chalk.default.bold('node_modules')} in your project folder.\n` + `  ${_chalk.default.cyan('3.')} Remove "${_chalk.default.bold(dep)}" from ${_chalk.default.bold('dependencies')} and/or ${_chalk.default.bold('devDependencies')} in the ${_chalk.default.bold('package.json')} file in your project folder.\n` + `  ${_chalk.default.cyan('4.')} Run ${_chalk.default.bold('yarn')}. \n\n` + `In most cases, this should be enough to fix the problem.\n` + `If this has not helped, there are a few other things you can try:\n\n` + `  ${_chalk.default.cyan('5.')} Check if ${_chalk.default.bold(maybeDep)} is outside your project directory.\n` + `     For example, you might have accidentally installed something in your home folder.\n\n` + `  ${_chalk.default.cyan('6.')} Try running ${_chalk.default.bold(`npm ls ${dep}`)} in your project folder.\n` + `     This will tell you which ${_chalk.default.underline('other')} package (apart from the expected ${_chalk.default.bold(ownPackageJson.name)}) installed ${_chalk.default.bold(dep)}.\n\n` + `If nothing else helps, add ${_chalk.default.bold('SKIP_PREFLIGHT_CHECK=true')} to an ${_chalk.default.bold('.env')} file in your project.\n` + `That would permanently disable this preflight check in case you want to proceed anyway.\n\n` + _chalk.default.cyan(`P.S. We know this message is long but please read the steps above :-)\n`));
        return false;
      }
    }
  }

  return true;
}
//# sourceMappingURL=verifyPackageTree.js.map