"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createPlugin;

var _formatError = require("../utils/formatError");

function createPlugin(name, server) {
  const plugin = {
    name: 'ws-reload',

    setup(build) {
      let building = false;
      let result;

      const publish = () => {
        // eslint-disable-next-line @typescript-eslint/no-misused-promises
        server.clients.forEach(async socket => {
          socket.send(JSON.stringify({
            name,
            building,
            result: {
              errors: await Promise.all(result.errors.map(_formatError.formatError)),
              warnings: await Promise.all(result.warnings.map(_formatError.formatError))
            }
          }));
        });
      };

      server.on('connection', () => {
        publish();
      });
      build.onStart(() => {
        building = true;
        publish();
      });
      build.onEnd(_result => {
        building = false;
        result = _result;
        publish();
      });
    }

  };
  return plugin;
}
//# sourceMappingURL=wsReload.js.map