"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = start;

var esbuild = _interopRequireWildcard(require("esbuild"));

var _chalk = _interopRequireDefault(require("chalk"));

var express = _interopRequireWildcard(require("express"));

var _expressWs = _interopRequireDefault(require("express-ws"));

var fs = _interopRequireWildcard(require("fs-extra"));

var path = _interopRequireWildcard(require("path"));

var tmp = _interopRequireWildcard(require("tmp"));

var _isCi = _interopRequireDefault(require("is-ci"));

var _memoize = _interopRequireDefault(require("../../utils/memoize"));

var _createPaths = _interopRequireDefault(require("../../utils/createPaths"));

var _getClientEnvironment = _interopRequireDefault(require("../config/getClientEnvironment"));

var _svgr = _interopRequireDefault(require("../plugins/svgr"));

var _incrementalCompile = _interopRequireDefault(require("../plugins/incrementalCompile"));

var _incrementalReporter = _interopRequireDefault(require("../plugins/incrementalReporter"));

var _wsReload = _interopRequireDefault(require("../plugins/wsReload"));

var _choosePort = _interopRequireDefault(require("../utils/choosePort"));

var _openBrowser = _interopRequireDefault(require("../utils/openBrowser"));

var logger = _interopRequireWildcard(require("../../utils/logger"));

var _urls = _interopRequireDefault(require("../config/urls"));

var _api = require("../api");

var _formatError = require("../utils/formatError");

var _createEsbuildConfig = _interopRequireDefault(require("../config/createEsbuildConfig"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class DevServer {
  constructor(paths) {
    _defineProperty(this, "started", false);

    _defineProperty(this, "_host", (0, _memoize.default)(() => {
      if (process.env.HOST) {
        logger.log(_chalk.default.cyan(`Attempting to bind to HOST environment variable: ${_chalk.default.yellow(_chalk.default.bold(process.env.HOST))}`));
        logger.log(`If this was unintentional, check that you haven't mistakenly set it in your shell.`);
        logger.log(`Learn more here: ${_chalk.default.yellow('https://cra.link/advanced-config')}`);
        logger.log();
      }

      return process.env.HOST || '0.0.0.0';
    }));

    _defineProperty(this, "urls", (0, _memoize.default)(async () => {
      return (0, _urls.default)(this.protocol, this.host, await this.port(), this.paths.publicUrlOrPath.slice(0, -1));
    }));

    _defineProperty(this, "port", (0, _memoize.default)(async () => {
      const port = await (0, _choosePort.default)(this.host, parseInt(process.env.PORT || '8000', 0));

      if (port) {
        return port;
      } else {
        throw new Error(`Could not identify port to run against`);
      }
    }));

    _defineProperty(this, "shudown", async () => {
      if (this.started) {
        var _this$server;

        (_this$server = this.server) === null || _this$server === void 0 ? void 0 : _this$server.close();
      }

      const esbuildServer = await this.esbuildServer();
      esbuildServer && esbuildServer.stop && esbuildServer.stop();
    });

    _defineProperty(this, "hostRuntime", (0, _memoize.default)(async () => {
      const runtimeDir = path.join(__dirname, '..', 'runtime');
      const runtimeDirFiles = await fs.readdir(runtimeDir);
      const indexFiles = runtimeDirFiles.filter(p => p.startsWith('index'));

      if (indexFiles.length !== 1) {
        throw new Error(`Found multiple possible entry files`);
      }

      const entryPoint = indexFiles[0];

      try {
        return await esbuild.build({
          entryPoints: [path.join(runtimeDir, entryPoint)],
          bundle: true,
          resolveExtensions: this.paths.moduleFileExtensions.map(extension => `.${extension}`),
          sourcemap: 'inline',
          absWorkingDir: this.paths.appPath,
          format: 'esm',
          target: 'es2015',
          logLevel: 'silent',
          color: true,
          define: {
            global: 'window'
          },
          watch: true,
          plugins: [(0, _wsReload.default)('runtime', this.ws.getWss())],
          outbase: runtimeDir,
          outdir: path.join(this.outdir, '_runtime'),
          publicPath: (await this.urls()).localUrlForBrowser
        });
      } catch (e) {
        const result = e;
        logger.log(_chalk.default.red('Failed to compile runtime.\n'));
        const logs = result.errors.concat(result.warnings).map(async m => {
          logger.log(await (0, _formatError.formatError)(m));
        });
        await Promise.all(logs);
        throw new Error(`Failed to compile runtime`);
      }
    }));

    _defineProperty(this, "runEsbuild", async watch => {
      const plugins = [(0, _svgr.default)(), (0, _incrementalReporter.default)()];
      let resolveIntialBuild;

      if (watch) {
        const {
          plugin,
          initialBuildPromise
        } = (0, _incrementalCompile.default)(this.paths, await this.urls());
        resolveIntialBuild = initialBuildPromise;
        plugins.push(plugin);
        plugins.push((0, _wsReload.default)('app', this.ws.getWss()));
      } else {
        resolveIntialBuild = Promise.resolve();
      }

      const result = await esbuild.build((0, _createEsbuildConfig.default)(this.paths, {
        plugins,
        watch,
        metafile: true,
        incremental: watch,
        minify: false,
        write: !watch,
        outdir: this.outdir
      })); // wait for the initial build to complete

      await resolveIntialBuild; // return the result of the build

      return result;
    });

    _defineProperty(this, "esbuildServer", (0, _memoize.default)(async () => {
      try {
        await this.runEsbuild(false);
      } catch (e) {
        const result = e;
        logger.log(_chalk.default.red('Failed to compile.\n'));
        const logs = result.errors.map(async m => {
          logger.log(await (0, _formatError.formatError)(m));
        });
        await Promise.all(logs);
        throw new Error(`Failed to compile`);
      } // if the non-incremental succeeds then
      // we start a watching server


      return this.runEsbuild(true);
    }));

    _defineProperty(this, "handleIndex", async (req, res) => {
      res.writeHead(200);
      res.end(await (0, _api.createIndex)(this.paths, this.env.raw, true));
    });

    this.paths = paths;
    this.env = (0, _getClientEnvironment.default)(paths.publicUrlOrPath.slice(0, -1));
    this.protocol = process.env.HTTPS === 'true' ? 'https' : 'http';
    this.outdir = tmp.dirSync().name;
    this.express = express.default();
    this.ws = (0, _expressWs.default)(this.express);
    const staticOptions = {
      cacheControl: false,
      index: false
    };
    this.express.use(express.static(this.outdir, staticOptions));
    this.express.use(express.static(paths.appPublic, staticOptions)); // eslint-disable-next-line @typescript-eslint/no-misused-promises

    this.express.get('/', this.handleIndex);
    this.ws.app.ws('/_ws', (ws, req) => {
      logger.debug('Connected');
    });
  }

  get host() {
    // wrap memoized version to prevent multiple logging calls.
    return this._host();
  }

  async start() {
    const port = await this.port(); // force clearing the terminal when we start a dev server process
    // unless we're in CI because we'll want to keep all logs

    logger.clear();
    logger.debug(`Using ${this.outdir}`);
    logger.log(_chalk.default.cyan('Starting the development server...\n')); // Start the esbuild before we startup the server

    await this.esbuildServer();
    await this.hostRuntime();
    return new Promise((resolve, reject) => {
      try {
        // eslint-disable-next-line @typescript-eslint/no-misused-promises
        this.server = this.express.listen(port, this.host, async () => {
          await (0, _openBrowser.default)((await this.urls()).localUrlForBrowser);
          this.started = true;
          resolve(this);
        });
      } catch (err) {
        logger.error(err);
        reject(err);
      }
    });
  }

}

async function start(target) {
  const paths = await (0, _createPaths.default)(target);
  const devServer = new DevServer(paths);
  const server = await devServer.start();
  ['SIGINT', 'SIGTERM'].forEach(sig => {
    process.on(sig, () => {
      void server.shudown();
    });
  });

  if (!_isCi.default) {
    // Gracefully exit when stdin ends
    process.stdin.on('end', () => {
      void server.shudown();
    });
  }
}
//# sourceMappingURL=index.js.map