"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = openBrowser;

var _chalk = _interopRequireDefault(require("chalk"));

var _child_process = require("child_process");

var _crossSpawn = _interopRequireDefault(require("cross-spawn"));

var _open = _interopRequireDefault(require("open"));

var logger = _interopRequireWildcard(require("../../utils/logger"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// https://github.com/sindresorhus/open#app
const OSX_CHROME = 'google chrome';
var Actions;

(function (Actions) {
  Actions["NONE"] = "None";
  Actions["BROWSER"] = "Browser";
  Actions["SCRIPT"] = "Script";
})(Actions || (Actions = {}));

const DEFAULT_BROWSER = process.env.BROWSER || OSX_CHROME;
const DEFAULT_BROWSER_ARGS = process.env.BROWSER_ARGS ? process.env.BROWSER_ARGS.split(' ') : []; // Will use the first open browser found from list

const SUPPORTED_CHROMIUM_BROWSERS = ['Google Chrome Canary', 'Google Chrome', 'Microsoft Edge', 'Brave Browser', 'Vivaldi', 'Chromium'];

function getBrowserEnv() {
  // Attempt to honor this environment variable.
  // It is specific to the operating system.
  // See https://github.com/sindresorhus/open#app for documentation.
  const value = DEFAULT_BROWSER;
  const args = DEFAULT_BROWSER_ARGS;
  let action;

  if (!value) {
    // Default.
    action = Actions.BROWSER;
  } else if (value.toLowerCase().endsWith('.js')) {
    action = Actions.SCRIPT;
  } else if (value.toLowerCase() === 'none') {
    action = Actions.NONE;
  } else {
    action = Actions.BROWSER;
  }

  return {
    action,
    value,
    args
  };
}

function executeNodeScript(scriptPath, url) {
  const extraArgs = process.argv.slice(2);
  const child = (0, _crossSpawn.default)(process.execPath, [scriptPath, ...extraArgs, url], {
    stdio: 'inherit'
  });
  child.on('close', code => {
    if (code !== 0) {
      logger.log();
      logger.log(_chalk.default.red('The script specified as BROWSER environment variable failed.'));
      logger.log(`${_chalk.default.cyan(scriptPath)} exited with code ${code}.`);
      logger.log();
      return;
    }
  });
  return true;
}

async function startBrowserProcess(targetBrowser, url, args) {
  // If we're on OS X, the user hasn't specifically
  // requested a different browser, we can try opening
  // Chrome with AppleScript. This lets us reuse an
  // existing tab when possible instead of creating a new one.
  const shouldTryOpenChromiumWithAppleScript = process.platform === 'darwin' && targetBrowser === OSX_CHROME;

  if (shouldTryOpenChromiumWithAppleScript) {
    for (const chromiumBrowser of SUPPORTED_CHROMIUM_BROWSERS) {
      try {
        // Try our best to reuse existing tab
        // on OSX Chromium-based browser with AppleScript
        (0, _child_process.execSync)('ps cax | grep "' + chromiumBrowser + '"');
        (0, _child_process.execSync)('osascript openChrome.applescript "' + encodeURI(url) + '" "' + chromiumBrowser + '"', {
          cwd: __dirname,
          stdio: 'ignore'
        });
        return true;
      } catch (err) {// Ignore errors.
      }
    }
  } // Another special case: on OS X, check if BROWSER has been set to "open".
  // In this case, instead of passing `open` to `opn` (which won't work),
  // just ignore it (thus ensuring the intended behavior, i.e. opening the system browser):
  // https://github.com/facebook/create-react-app/pull/1690#issuecomment-283518768


  if (process.platform === 'darwin' && targetBrowser === 'open') {
    targetBrowser = undefined;
  } // Fallback to open
  // (It will always open new tab)


  try {
    await (0, _open.default)(url, {
      app: targetBrowser ? {
        name: targetBrowser,
        arguments: args
      } : undefined,
      wait: false
    });
    return true;
  } catch (err) {
    return false;
  }
}
/**
 * Reads the BROWSER environment variable and decides what to do with it. Returns
 * true if it opened a browser or ran a node.js script, otherwise false.
 */


async function openBrowser(url) {
  const {
    action,
    value,
    args
  } = getBrowserEnv();

  switch (action) {
    case Actions.NONE:
      // Special case: BROWSER="none" will prevent opening completely.
      return false;

    case Actions.SCRIPT:
      return executeNodeScript(value, url);

    case Actions.BROWSER:
      return startBrowserProcess(value, url, args);

    default:
      throw new Error('Not implemented.');
  }
}
//# sourceMappingURL=openBrowser.js.map