"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getAllWorkspaces = void 0;
exports.getYarnWorkspaceInfo = getYarnWorkspaceInfo;

var _execa = _interopRequireDefault(require("execa"));

var _memoize = _interopRequireDefault(require("./memoize"));

var _getModularRoot = _interopRequireDefault(require("./getModularRoot"));

var logger = _interopRequireWildcard(require("./logger"));

var _stripAnsi = _interopRequireDefault(require("strip-ansi"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

async function getYarnWorkspaceInfo(cwd) {
  const result = await (0, _execa.default)('yarnpkg', ['--silent', 'workspaces', 'info'], {
    cwd,
    reject: false,
    cleanup: true
  });
  const {
    exitCode,
    stdout,
    stderr
  } = result;

  if (stderr && !!exitCode) {
    logger.error(stderr);
    throw new Error(`Failed to lookup yarn workspace information`);
  } // strip out ANSI color codes and escape characters


  const strippedStd = (0, _stripAnsi.default)(stdout // there's an edge case where if there are no packages in the current workspace
  // then this command returns an empty string and no JSON - so we have to default
  // to an empty directory which can be JSON parsed.
  ) || '{}';
  return JSON.parse(strippedStd);
}

function _getAllWorkspaces() {
  const modularRoot = (0, _getModularRoot.default)();
  return getYarnWorkspaceInfo(modularRoot);
}

const getAllWorkspaces = (0, _memoize.default)(_getAllWorkspaces);
exports.getAllWorkspaces = getAllWorkspaces;
//# sourceMappingURL=getAllWorkspaces.js.map