"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.findModularRoot = exports.default = void 0;

var path = _interopRequireWildcard(require("path"));

var fs = _interopRequireWildcard(require("fs-extra"));

var _findUp = _interopRequireDefault(require("find-up"));

var _memoize = _interopRequireDefault(require("./memoize"));

var logger = _interopRequireWildcard(require("./logger"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function isModularRoot(packageJsonPath) {
  var _packageJson$modular;

  const packageJson = fs.readJSONSync(packageJsonPath, {
    encoding: 'utf8'
  });
  return (packageJson === null || packageJson === void 0 ? void 0 : (_packageJson$modular = packageJson.modular) === null || _packageJson$modular === void 0 ? void 0 : _packageJson$modular.type) === 'root';
} // This function is not memoized and gets executed at most twice:
// when it's accessed directly (at init time) and when it's accessed via getModularRoot (at command time)
// The reason for this is that we might want to refresh the modular root
// in case we run on a non-modular directory to upgrade to a modular one,
// on which we still need to do checks (see for example the "convert" command)


const findModularRoot = function findModularRoot() {
  try {
    logger.debug('Deferring to find-up to locate modular root');

    const modularRoot = _findUp.default.sync(directory => {
      const packageJsonPath = path.join(directory, 'package.json');

      if (_findUp.default.sync.exists(packageJsonPath) && isModularRoot(packageJsonPath)) {
        return packageJsonPath;
      }

      return;
    }, {
      type: 'file',
      allowSymlinks: false
    });

    return modularRoot ? path.normalize(path.dirname(modularRoot)) : undefined;
  } catch (err) {
    throw new Error(err);
  }
};

exports.findModularRoot = findModularRoot;

function getModularRoot() {
  const modularRoot = findModularRoot();

  if (modularRoot === undefined) {
    throw new Error('Could not find modular root.');
  }

  logger.debug(`Located modular root ${modularRoot}`);
  return modularRoot;
}

var _default = (0, _memoize.default)(getModularRoot);

exports.default = _default;
//# sourceMappingURL=getModularRoot.js.map